package xbeans;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.*;

import org.w3c.dom.Document;
import org.xml.sax.*;


import java.io.*;
import java.util.*;




/** Creates an XSLT transformer for processing an XML document.
 *  A new transformer, along with an style template are created 
 *  for each document transformation. The XSLT, DOM, and 
 *  SAX processors are based on system default parameters.
 *
 *  Taken from Core Web Programming from 
 *  Prentice Hall and Sun Microsystems Press,
 *  http://www.corewebprogramming.com/.
 *  &copy; 2001 Marty Hall and Larry Brown;
 *  may be freely used or adapted. 
 */ 

public class XslTransformer {
  private TransformerFactory factory;

  public XslTransformer() {


	if(System.getProperty("javax.xml.transform.TransformerFactory") == null) {
		System.setProperty("javax.xml.transform.TransformerFactory", "com.icl.saxon.TransformerFactoryImpl");
	}


    factory =  TransformerFactory.newInstance();
  }

  /** Transform an XML and XSL document as <code>Reader</code>s,
   *  placing the resulting transformed document in a 
   *  <code>Writer</code>. Convenient for handling an XML 
   *  documents as a String (<code>StringReader</code>) residing
   *  in memory, not on disk. The output document could easily be
   *  handled as a String (<code>StringWriter</code>) or as a
   *  <code>JSPWriter</code> in a JavaServer page.
   */

  public void process(Reader xmlFile, Reader xslFile,
                      Writer output)
                throws TransformerException {
    process(new StreamSource(xmlFile),
            new StreamSource(xslFile),
            new StreamResult(output));
  }

  /** Transform an XML and XSL document as <code>File</code>s,
   *  placing the resulting transformed document in a 
   *  <code>Writer</code>. The output document could easily 
   *  be handled as a String (<code>StringWriter</code)> or as 
   *  a <code>JSPWriter</code> in a JavaServer page.
   */

  public void process(File xmlFile, File xslFile,
                      Writer output)
                throws TransformerException {
    process(new StreamSource(xmlFile),
            new StreamSource(xslFile),
            new StreamResult(output));
  }

  /** Transform an XML <code>File</code> based on an XSL 
   *  <code>File</code>, placing the resulting transformed 
   *  document in an <code>OutputStream</code>. Convenient for 
   *  handling the result as a <code>FileOutputStream</code> or 
   *  <code>ByteArrayOutputStream</code>.
   */

  public void process(File xmlFile, File xslFile,
                      OutputStream out)
                 throws TransformerException {
    process(new StreamSource(xmlFile),
            new StreamSource(xslFile),
            new StreamResult(out));
  }

  /** Transform an XML source using XSLT based on a new template
   *  for the source XSL document. The resulting transformed 
   *  document is placed in the passed in <code>Result</code> 
   *  object.
   */

  public void process(Source xml, Source xsl, Result result)
                throws TransformerException {
    try {
      factory.setErrorListener(new MyErrorHandler(System.err));

      Templates template = factory.newTemplates(xsl);
      Transformer transformer = template.newTransformer();
      // Set an ErrorHandler before parsing
      transformer.transform(xml, result);

	  System.out.println(" done txing returning ");
      

    } catch(TransformerConfigurationException tce) {

        throw new TransformerException(
                    tce.getMessageAndLocation());
    } catch (TransformerException te) {
		System.out.println(" ############msg loc =########### " + 	te.getMessageAndLocation());
      throw new TransformerException(
                  te.getMessage());
    }
  }


  	public static void main(String[] args) {
  		
		Document xmldoc = null;
		Document xsldoc = null;
		Document xsddoc = null;

  		XslTransformer xformer = new XslTransformer();
  		
		 try {
            DocumentBuilderFactory factory =
                DocumentBuilderFactory.newInstance();
             factory.setIgnoringElementContentWhitespace(true);
               DocumentBuilder builder = factory.newDocumentBuilder();
               
               xmldoc = builder.parse( new File(args[0]) );

               xsddoc = builder.parse( new File(args[1]) );
               
               xsldoc = builder.parse( new File(args[2]) );

				System.out.println(" fin parsing ");
               
            
    	Source xmlSource = new javax.xml.transform.dom.DOMSource(xmldoc);
		
		Source xslSource = new javax.xml.transform.dom.DOMSource(xsldoc);

		    			/* Prepare the result */
		DOMResult result = new DOMResult();
		
		/* Traverse the DOM tree */
		xformer.process(xmlSource, xslSource, result);

		Document doc2 = (Document) result.getNode();
		
		String condition = doc2.getDocumentElement().getFirstChild().getNodeValue();

		System.out.println(" rule cond " + condition);
	
            } catch(TransformerException te) {
         	te.printStackTrace();
         	
            }
            catch (SAXParseException spe) {
        /* Error generated by the parser */
      /*  System.out.println("\n** Parsing error"
           + ", line " + spe.getLineNumber()
           + ", uri " + spe.getSystemId());
		
        System.out.println("   " + spe.getMessage() );
		*/
        /* Use the contained exception, if any */
        Exception  x = spe;
        if (spe.getException() != null)
            x = spe.getException();
        x.printStackTrace();

            } catch (SAXException sxe) {
                /* Error generated during parsing) */
                Exception  x = sxe;
                if (sxe.getException() != null)
                    x = sxe.getException();
                x.printStackTrace();

            } catch (ParserConfigurationException pce) {
                 /* Parser with specified options can't be built */
                 pce.printStackTrace();
             } catch (IOException ioe) {
                ioe.printStackTrace();
             } 

  	
  	}
  

    // Error handler to report errors and warnings
    private static class MyErrorHandler implements ErrorListener{
        /** Error handler output goes here */
        private PrintStream out;

        MyErrorHandler(PrintStream out) {
            this.out = out;
        }

        /**
         * Returns a string describing parse exception details
         */
        private String getParseExceptionInfo(TransformerException spe) {

			String info = null;
			Throwable innerex = spe.getException();
			
			if(innerex != null) {

				if(innerex instanceof org.xml.sax.SAXParseException) {
				
					org.xml.sax.SAXParseException spinner = (org.xml.sax.SAXParseException) innerex;

		            String systemId = spinner.getSystemId();
		            if (systemId == null) {
				        systemId = "null";
					}
					
					info = "URI=" + systemId +
								" Line=" + spinner.getLineNumber() + 
								": " + spinner.getMessage();	
					return info;
				}
			} 

			if(spe.getLocator() != null) {

				String systemId = spe.getLocator().getSystemId();
				//String systemId = null;
				if (systemId == null) {
					systemId = "null";
				}
				
				info = "URI=" + systemId +
					" Line=" + spe.getLocator().getLineNumber() + 
					": " + spe.getMessage();	

			}
			
            return info;
        }

        // The following methods are standard SAX ErrorHandler methods.
        // See SAX documentation for more info.

        public void warning(TransformerException spe) throws TransformerException {
            out.println("Warningggg: " + getParseExceptionInfo(spe));
		    throw new TransformerException(getParseExceptionInfo(spe));
       
        }
        
        public void error(TransformerException spe) throws TransformerException {
            String message = "Errorrrrrrrrr: " + getParseExceptionInfo(spe);
		    System.out.println("Error: " + message);
            throw new TransformerException(message);
        }

        public void fatalError(TransformerException spe) throws TransformerException {
            String message = "Fatal Errorrrrrrrrrr: " + getParseExceptionInfo(spe);
		    System.out.println("fatalError: " + message);
		    throw new TransformerException(message);


        }
    }


}