
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;


/**

 * This class is a wrapper over the log4j logging system.
 * Log4j supports hierarchy and level based logging.
 * Hierarchy based logging is acheived by building a hierarchy of loggers,
 * ideally one per module, which allows selective enabling or disabling
 * of logging for a module.Level based logging is acheived through four
 * levels of logging, namely DEBUG, INFO, WARN, ERROR and FATAL.
 * DEBUG is the lowest level and FATAL is the highest level.
 * This class exposes methods to set the logging leves,
 * to set the message category and to build a hierarchy of loggers
 *
 * Usage:
 *
 * CASSLogger cassLogger = CASSLogger.getLogger(MODULE_LOGGER); (Returns a logger for the module)
 *
 * logger.debug(""); 	//For a messages belonging to debug category
 * logger.info(""); 	//For a messages belonging to information category
 * logger.warn(""); 	//For a messages belonging to warinig category
 * logger.error(""); 	//For a messages belonging to error category
 * logger.fatal(""); 	//For a messages belonging to fatal category
 *
 * NOTE: The path of the configuration file
 * should be set in a "paths.properties" file against key Log4jConfigFile
 * example
 *
 * Log4jConfigFile=e:/log4j/Log4jConfig.xml
 *
 * or
 *
 * Log4jConfigFile=/cpkbuild/users/cpkbld1/checkin/common/xml/Log4jConfig.xml
 *
 * @author Polaris Software Lab
 * @version 1.0
 */
public class BRELogger
{
	/**
	 * Root logger
	 */
	private static Logger logger= null;
	/**
	 * Logger cache
	 */
	private static Map cache= new HashMap();
	/**
	 * Module logger
	 */
	private Logger modLogger= null;
	/**
	 * Printing stack trace
	 */
	private static boolean printStackTrace= true;
	/**
	 * Static block to initialize the static variables
	 */
	/* static {
		ResourceBundle rb= ResourceBundle.getBundle("environment");
		String loggerpath= rb.getString("Log4jConfigFile");
		String stackTrace= rb.getString("PrintStackTrace");
		PropertyConfigurator.configure(loggerpath);
		logger= Logger.getRootLogger();
		logger.setLevel((Level) Level.INFO);
		cache= Collections.synchronizedMap(new HashMap());
		if (stackTrace.equals("true"))
		{
			printStackTrace= true;
		}
		else
		{
			printStackTrace= false;
		}
	} */
	/**
	 * @param moduleLogger
	 */
	private BRELogger(Logger moduleLogger)
	{
		modLogger= moduleLogger;
	}
	/**
	 * Default Constructor
	 * 
	 */
	private BRELogger()
	{
	}
	/**
	 * Factory method for module logger
	 * @param moduleName - Name of the module for which logger is required
	 * @return Instance of this classes wrapping the module logger
	 */
	public static BRELogger getLogger(String moduleName)
	{
		//System.out.println("moduleName :: "+moduleName);
		Logger moduleLogger= (Logger) cache.get(moduleName);
		//System.out.println(" moduleLogger ::"+moduleLogger);
		if (moduleLogger == null)
		{
			moduleLogger= Logger.getLogger(moduleName);
			cache.put(moduleName, moduleLogger);
		}
		return new BRELogger(moduleLogger);
	}
	/**
	 * Causes all categories of messages to be logged.
	 *
	 */
	public void setInfoLevel()
	{
		modLogger.setLevel((Level) Level.INFO);
	}
	/**
	 * Only debug, warn, error and fatal messages get logged.
	 *
	 */
	public void setDebugLevel()
	{
		modLogger.setLevel((Level) Level.DEBUG);
	}
	/**
	 * Only warn, error and fatal messages get logged.
	 *
	 */
	public void setWarnLevel()
	{
		modLogger.setLevel((Level) Level.WARN);
	}
	/**
	 *Only error and fatal messages get logged.
	 *
	 */
	public void setErrorLevel()
	{
		modLogger.setLevel((Level) Level.ERROR);
	}
	/**
	 * Only fatal messages get logged.
	 *
	 */
	public void setFatalLevel()
	{
		modLogger.setLevel((Level) Level.FATAL);
	}
	/**
	 * Logs the message if the level is set at DEBUG
	 * @param message - message to be logged
	 */
	public void debug(String message)
	{
		modLogger.debug(message);
	}
	/**
	 * Logs the message if the level is set at INFO or lower
	 * @param obj
	 * @param throwable
	 */
	public void info(String message)
	{
		modLogger.info(message);
	}
	/**
	 * Logs the message if the level is set at WARN or lower
	 * @param obj
	 */
	public void warn(String message)
	{
		modLogger.warn(message);
	}
	/**
	 * Logs the message if the level is set at ERROR or lower
	 * @param message
	 * @param throwable
	 */
	public void error(Object obj, Throwable throwable)
	{
		modLogger.error(obj, throwable);
	}
	/**
	 * Logs the message if the level is set at FATAL or lower
	 * @param obj - Object to be rendered
	 * @param throwable - Exception object to be rendered
	 */
	public void fatal(Object obj, Throwable throwable)
	{
		modLogger.fatal(obj, throwable);
	}
	/**
	 * Returns the current level of logging
	 * @return Current level of logging
	 */
	public String getLevel()
	{
		return modLogger.getLevel().toString();
	}
	/**This method is used to print stack trace of an exception
	 * @param exception
	 */
	public void printTrace(Exception e)
	{
		if (printStackTrace)
		{
			modLogger.fatal("Error", e);
		}
	}
    
	public static void log(String module,String message)
	{
		/*BRELogger logger= BRELogger.getLogger(module);
		logger.setDebugLevel();
		logger.debug(message);*/
		System.out.println(module + ": " + message);
	}
}
